#!/bin/sh

#
# hier.sh - affiche la date d'hier
#
# Usage : hier.sh [ JJ MM AAAA | JJ/MM/AAAA ]
#
# Description :
# hier.sh affiche la date d'hier ou de la veille du jour passe en argument.
# La date passee en argument peut avoir le format JJ MM AAAA ou JJ/MM/AAAA.
# Dans tous les cas, la date est affichee au format JJ/MM/AAAA.
#
# Options :
# JJ le jour du mois (01..31),
# MM le mois (01..12),
# AAAA represente l'annee (1970...).
#

if [ $# -eq 1 ]
then
    _IFS="$IFS"
    IFS="/"
    set -- $@
    IFS="$_IFS"
elif [ $# -eq 3 ]
then
    set -- $@
else
    set -- `date "+%d %m %Y"`
    #set -- `date "+%Y %m %d"`
fi

if [ $1 -eq 1 ] # premier jour du mois
then
    case "$2" in
        01)                # bonne annee !
            Annee=`expr $3 - 1`
            Mois=12
            Jour=31
            ;;
        03)                # mois precedent = fevrier
            Annee=$3
            Mois=02
            [ `expr $3 % 4` -eq 0 ] && Jour=29 || Jour=28
            ;;
        02|04|06|08|09|11) # mois precedent de 31 jours
            Annee=$3
            Mois=`expr $2 - 1`
            Jour=31
            ;;
        05|07|10|12)       # mois precedent de 30 jours
            Annee=$3
            Mois=`expr $2 - 1`
            Jour=30
            ;;
    esac
else # hier c'est aujourd'hui moins 1
    Annee=$3
    Mois=$2
    Jour=`expr $1 - 1`
fi

echo "$Mois" | grep "^[0-9]$" > /dev/null && Mois="0$Mois"
echo "$Jour" | grep "^[0-9]$" > /dev/null && Jour="0$Jour"

echo "$Jour/$Mois/$Annee"
#echo "$Annee$Mois$Jour"

