#!/bin/sh

#
# sendmail.sh - programme appelle par le MUA comme suit :
#     cat message | sendmail.sh
#
# sendmail.sh doit donc lire sur l'entree standard le mel, et faire, pour
# chaque destinataire, un envoi au serveur concerne
#

cd `dirname $0`

Params="$@"

[ -f ./commun-smtpd ] && . ./commun-smtpd
[ -z "$DaemonMailer" ] && DaemonMailer="Postmaster"
[ -z "$PORT" ] && PORT=25
[ -z "$Tmp" ] && Tmp=/tmp/minsmtpd.$$
[ -z "$FicLog" ] && FicLog="/tmp/minsmtpd.log"
[ -z "$Queue" ] && Queue="/tmp/spool/mqueue"
[ -z "$Fic" ] && Fic=`expr \`date +%s\` % 1000`
[ -z "$Hostname" ] && Hostname=`hostname -s`
[ -z "$Domaine" ] && Domaine="`hostname -d`"
[ -z "$Fqdn" ] && Fqdn="$Hostname.$Domaine"
[ -z "$Log" ] && Log="[`basename $0`] `date`"

[ -e ./commun-smtpd.local ] && . ./commun-smtpd.local

if ! ChercheLoop > /dev/null 2>&1
then
    ChercheLoop ()
    {
        return 1
    }
fi

export PATH=.:$PATH

rm -rf $Tmp

echo >> $FicLog
date >> $FicLog

#
# Premiere partie : analyse du mel pour en extraire les adresses de
# l'expediteur (champ From:), des destinataires (champ To:, Cc: ou Bcc:).
# L'adresse de l'expediteur va nous servir par la suite pour envoyer un
# mel d'erreur a l'expediteur si une adresses du (des) destinataire(s)
# est non valide.
# L'adresse des destinataires va bien evidemment servir a contacter les
# serveurs de mels en charge de leur(s) domaine(s)
#

cat > $Tmp
cat $Tmp | while read -r ligne
do

#
# On est dans le corps du message. Cette info est utile car les destinataires
# du message ne doivent pas etre pris dans le corps du message (par exemple
# pour un message transmis par un utilisateur, le corps du message contient
# ce genre d'infos).
#

    [ -z "$ligne" ] && break
    set -- $ligne
    Cmd=`echo $1 | tr [a-z] [A-Z]`
    case "$Cmd" in 
        "FROM:")
            while [ -n "$2" ]
            do
                #echo -n "$1 " >> $Tmp
                if echo "$2" | grep "@" > /dev/null
                then
                    From=`echo $2 | sed -e "s/.*<//g" -e "s/>.*//g"`
                    User=`echo $From | cut -d @ -f1`
                    if [ -f genericstable ]
                    then
                        set -- `grep "^$User" genericstable | tr ":" " "`
                    fi
                    if [ -n "$2" ]
                    then
                        From="<$2>"
                        #echo -n "<$2>" >> $Tmp
                        if [ -f genericstable ]
                        then
                            echo "$Log genericstable $1 -> $2" >> $FicLog
                        fi
                    fi
                    break
                else
                    shift
                fi
            done
            #echo >> $Tmp
            ;;
        "TO:"|"CC:"|"BCC:")
            TraiteTo ()
            {
                ligne="$@"
                set -- $ligne
                #echo -e "$ligne" >> $Tmp
                if [ $# -lt 2 ] # exemple : user@domaine
                then
                    To="$To `echo $1 | sed -e "s/.*<//g" \
                                                 -e "s/>.*//g" \
                                                 -e "s/,//g"`"

                else # exemple : Nom Prenom <user@domaine>
                    while [ -n "$2" ]
                    do
                        if echo "$2" | grep "@" > /dev/null
                        then
                            To="$To `echo $2 | sed -e "s/.*<//g" \
                                                 -e "s/>.*//g" \
                                                 -e "s/,//g"`"
                        fi
                        shift
                    done
                fi
            }

#
# Est-on dans l'en-tete du message ?
#

#
# Oui -> traitement des destinataires du message
#

                TraiteTo "$ligne"
                while read ligne2
                do
                    ligne2=`echo "$ligne2" |
                            sed -e "s/\[:space:\]/ /g" -e "s/\[:tab:\]/	/g"`
                    if echo "$ligne2" | egrep ": |^$" | egrep -v "^To: |^Cc: |^Bcc: " > /dev/null
                    then
                        #echo "$ligne2" >> $Tmp # BUG si la ligne est From:
                        break
                    fi
                    TraiteTo "$ligne2"
                done

#
# Non -> la ligne est traitee comme une ligne du coprs du message
#

            ;;
        *) ;; #echo "$ligne" >> $Tmp ;;
    esac
    echo "
From $From
User $User
To $To" > $Tmp.$$
done

From=`grep "^From " $Tmp.$$ | cut -d " " -f2`
[ -z "$From" ] && From="$LOGNAME@$Fqdn"
User=`grep "^User " $Tmp.$$ | cut -d " " -f2`

#
# Le corps du message est mis dans le fichier temporaire
#

#echo >> $Tmp
#sed "/^From /,/^$/d" >> $Tmp

#cp $Tmp /tmp/mail

#
# Si les destinataires sont passes en arguement au script, on les prend.
# Sinon, on les recuperes dans le message.
#

if [ -n "$Params" ]
then
    for param in $Params
    do
        if echo "$param" | grep "@" > /dev/null
        then
            To="$To $param"
        elif grep "^$param:" /etc/passwd > /dev/null
        then
            To="$To $param@localhost"
        fi
    done
fi

[ -z "$To" ] && To=`grep "^To " $Tmp.$$ | sed "s/^To *//"`

rm -f $Tmp.$$

#
# Deuxieme partie : pour chaque destinataire, trouver quel serveur
# contacter, envoyer un mel d'erreur a l'expediteur si on ne trouve
# pas de serveur, et envoyer le mel via une connexion SMTP
#

echo "$Log All dest: $To" >> $FicLog
for dest in $To
do

#    echo "$Log To: $dest" >> $FicLog

#
# Quel serveur (MX) contacter ?
#

    Mx=""

#
# Quel est le nom de domaine du destinataire (partie a droite du '@'
# dans l'adresse). Par exemple, mdecore@chez.moi a pour nom de domaine
# chez.moi
#

    To=`echo $dest | sed "s/@.*//"`
    Dest=`echo $dest | sed "s/.*@//"`
#    echo "$Log To: $Dest" >> $FicLog

    case "$Dest" in

#
# serveur = machine locale -> directement mis dans la BAL
#

        localhost|$Hostname|$Fqdn)
            echo "$Log Trying local $Dest:$PORT" >> $FicLog
            if [ -d /var/mail ]
            then
                MailBox=/var/mail/$To
            elif [ -d /var/spool/mail ]
            then
                MailBox=/var/spool/mail/$To
            else
                echo "$Log ERROR rep for mailbox found - exit." >> $FicLog
                exit 1
            fi

#
# Creation de la boite aux lettres si necessaire et mise a jours des droits
#

            if [ ! -f $MailBox ]
            then
                : > $MailBox
                chmod 600 $MailBox
                Group=`grep "$To" /etc/passwd | cut -d : -f4`
                Group=`grep "^[^:]*:[^:]*:$Group:" /etc/group | cut -d : -f1`
                chown $To:$Group $MailBox
            fi

            From=`echo $From | sed -e "s/^<//" -e "s/>$//"`
            From=`echo $From | sed "s/\([^@]*\)@.*/\1/"`
            unset LANG
            echo "From $From@$Fqdn `date`" >> $MailBox
            sed -e "/^Bcc:/d" < $Tmp >> $MailBox
            echo >> $MailBox
            echo "$Log END OK" >> $FicLog
#                rm -rf $Tmp
            continue
            ;;

#
# Ce domaine a t-il des serveurs de mels (MX) definits par le DNS ?
# Par exemple, le domaine chez.moi a pour MX tarsier.chez.moi
#

        *) ListeMx=`host -t mx $Dest 2> /dev/null | sort -n | cut -d" " -f7` ;;
    esac

#
# Si oui, ces serveurs sont-ils interrogables (ie. on peut etablire une
# connexion sur le port SMTP) -> envoi d'une trame ICMP pour tester. On
# repete l'operation pour tous les serveurs, classes precedemment par ordre
# de preference, jusqu'a ce qu'il y en ai un qui reponde.
# Si aucun serveur MX n'est definit, on prend le nom de domaine de l'adresse
# du destinataire (en esperant qu'elle corresponde a une machine)
#

#    for serveur in $ListeMx $Dest
#    do
#        case "$serveur" in
#
#            localhost|$Hostname|$Fqdn)
#        esac
#    done

    for serveur in $SmartHost $ListeMx $Dest
    do
        if echo "$serveur" | grep ":" > /dev/null
        then
            PORT=`echo $serveur | cut -d: -f2`
            serveur=`echo $serveur | cut -d: -f1`
        fi
        echo "$Log Trying $serveur:$PORT" >> $FicLog
        if nc -w 5 -z $serveur $PORT > /dev/null 2>&1
        then
            sleep 1
            Mx=$serveur
            echo "$Log $serveur:$PORT OK." >> $FicLog
            break
        else
            echo "$Log $serveur:$PORT failed." >> $FicLog
            continue
        fi
    done

#
# Le serveur de mels du destinataire est-il definit (ie. il a un nom
# valide est peut etre contacte) ?
#

    if [ -n "$Mx" ]
    then
        echo "$Log $dest -> $Mx:$PORT" >> $FicLog

#
# Si oui, on envoie le mel au destinataire
#

        (
        echo -e "helo $Fqdn\r"
        sleep 2
        echo -e "mail from: $From\r"
        sleep 2
        echo -e "rcpt to: <$dest>\r"
        sleep 2
        echo -e "data\r"
        sleep 2
        sed -e "/^Bcc:/d" -e "s/$/
/" < $Tmp
        #sed -e "/^Bcc:/d" < $Tmp | ./lf2crlf
        sleep 2
        echo -e ".\r"
        sleep 2
        echo -e "quit\r"
        sleep 2
        ) | nc -w 30 $Mx $PORT > $Tmp.$$
        [ $? -ne 0 ] && echo "$Log Erreur envoi mail $Mx:$PORT" >> $FicLog
        grep "^5" $Tmp.$$ | sed "s/^/$Log/" >> $FicLog 2>&1
        rm -f $Tmp.$$

#
# Sinon, on envoie un mel d'erreur a l'expediteur
#

            elif ChercheLoop && [ -n "$SendErrorMails" ]
            then
                echo "$Log Domaine inconnu : $dest" >> $FicLog
                (cat << EOF
From: $DaemonMailer@$Domaine
To: $From
Date: `date`
Subject: Unvalid domain
X-Loop: $DaemonMailer@$Domaine

EOF
                (sed -e "s/TO/$dest/" \
                     -e "s/DOMAINE/`echo $dest | sed "s/.*@//"`/") \
                < $Format/domaine-non-valide
                cat $Tmp
                ) > $Queue/$Fic
            else
                rm -rf $Tmp
                echo "$Log END KO" >> $FicLog
                exit 1
    fi # [ -n "$Mx" ]
    sleep 1
done

echo "$Log END OK" >> $FicLog

rm -rf $Tmp

